--[[
	This script translates BrainFuck code.

	this can do these syntax symbols
	>
	<
	+
	-
	[
	]
	.
]]

--Deleted some unneeded variables

local subs = {
    ---Wrap cell values to 256 so that string.char doesn't error
	["+"] = "cells[currentCell] = ((cells[currentCell] or 0) + 1) % 256",
	["-"] = "cells[currentCell] = ((cells[currentCell] or 0) - 1) % 256",
    --Instead of directly printing we write to an output string
	["."] = "output = output .. string.char(cells[currentCell] or 0))",
	[">"] = "currentCell = (currentCell + 1) % 30000",
	["<"] = "currentCell = (currentCell - 1) % 30000",
	["["] = "while cells[currentCell] ~= 0 do",
	["]"] = "end",
}

--This function is used to prevent infinite loops
local function timeup ()
    debug.sethook()
    error("Script timed up", 2)
end

--We just need to declare compile once and not every time we call interpret
local function compile(source)
    local result = {}

    for char in source:gmatch(".") do
        if subs[char] then
            table.insert(result, subs[char])
        else --Lua line number = Brainfuck character
            table.insert(result, "")
        end
    end

    --At the end we return the output string
    table.insert(result, "return output")

    local f, err = loadstring(table.concat(result, "\n"))

    --Compilation failed!
    if not f then
        return false, err
    end

    --We limit the program scope to this table
    local env = {
        --This are the only variables needed by the program
        currentCell = 1,
        cells = {},
        output = "",

        --string.char
        string = {char = string.char}
    }

    setfenv(f, env)

    local run = function (quota, ...)
        --How many instructions can the program use
        quota = quota or 10000
        --If the number of instruction is exceeded call timeup
        debug.sethook(timeup, "", quota)
        --Run the program and return it's output
        local value = f(...)
        debug.sethook()
        return value
    end

    --We return true, compilation succeeded
    --A secure way to run the program
    --And the environment where the program runs
    return true, function (...) return pcall(run, ...) end, env
end

local function interpret (bfStr)
	local success, program = compile(bfStr)

	if success then --Compilation succeded
        local output
		success, output = program()

        if success then --Execution succeeded
            print(output)
        else --Execution failed
            print("INTERPRETER ERROR: " .. output)
        end
	else --Compilation failed
		print("COMPILER ERROR: " .. program)
		return
	end
end

function love.update()
	interpret(io.read())
end
